package org.apache.xindice.core.meta;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: TimeRecord.java,v 1.1 2002/12/27 18:37:24 dviner Exp $
 */

import java.util.Date;
import java.util.Hashtable;
/**
 * Encapsulates the modified/create times used in MetaData.
 * 
 */
public class TimeRecord
{
   long created = 0;
   long modified = 0;


   public TimeRecord( long created, long modified )
   {
      this.created = created;
      this.modified = modified;
   }

   /**
    * Returns the create time stored here.
	*
	* @return long
    */
   public long getCreatedTime()
   {
      return created;
   }

   /**
    * Returns the modified time stored here.
	*
	* @return long
    */
   public long getModifiedTime()
   {
      return modified;
   }

   /**
    * Sets the create time stored here.
	*
	* @param created The new create time
    */
   public void setCreatedTime(long created)
   {
	   this.created = created;
   }

   /**
    * Sets the modified time stored here.
	*
	* @param mod The new modified time
    */
   public void setModifiedTime(long mod)
   {
	   this.modified = mod;
   }

   /**
    * Return a string version of the times encapsulated. It should look like
	*   [cr=CREATE mod=MOD]
	*
	* @return The string representing the time
    */
   public String toString()
   {
      StringBuffer buffer = new StringBuffer();
      buffer.append("[");
      if( created > 0 )
         buffer.append("cr=" + (new Date(created)).toString());
      if( modified > 0 )
         buffer.append("mod=" + (new Date(modified)).toString());
      buffer.append("]");
      return buffer.toString();
   }
}
