/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: XmlDbClient.java,v 1.3 2002/10/28 08:39:20 vladimir Exp $
 */

package org.apache.xindice.integration.client;

import org.apache.xindice.client.xmldb.services.CollectionManager;
import org.apache.xindice.xml.dom.DOMParser;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.modules.XMLResource;

/**
 * @version $Revision: 1.3 $, $Date: 2002/10/28 08:39:20 $
 * @author Vladimir R. Bossicard <vladimir@apache.org>
 */
public class XmlDbClient {

   protected String driver;

   public XmlDbClient(String driver)
         throws Exception {
      this.driver = driver;
   }

   public String getName(String path)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      return col.getName();
   }

   public Collection createCollection(String path,
                                      String name)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      CollectionManager service = (CollectionManager) col.getService("CollectionManager", "1.0");

      String collectionConfig = "<collection compressed=\"true\" name=\"" + name + "\">" +
            "   <filer class=\"org.apache.xindice.core.filer.BTreeFiler\" gzip=\"true\"/>" +
            "</collection>";
      return service.createCollection(name, DOMParser.toDocument(collectionConfig));
   }

   public Collection getCollection(String path)
         throws Exception {
      return DatabaseManager.getCollection(driver + "/" + path);
   }

   public void dropCollection(String path,
                              String name)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      CollectionManager service = (CollectionManager) col.getService("CollectionManager", "1.0");

      service.dropCollection(name);
   }

   public String[] listCollections(String path)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      return col.listChildCollections();
   }

   public int countCollections(String path)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      return col.getChildCollectionCount();
   }

   public void createIndexer(String path,
                             String name,
                             String indexdef)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      CollectionManager service = (CollectionManager) col.getService("CollectionManager", "1.0");

      service.createIndexer(DOMParser.toDocument(indexdef));
   }

   public String[] listIndexes(String path)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      CollectionManager service = (CollectionManager) col.getService("CollectionManager", "1.0");

      return service.listIndexers();
   }

   public void dropIndexer(String path,
                           String name)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      CollectionManager service = (CollectionManager) col.getService("CollectionManager", "1.0");

      service.dropIndexer(name);
   }

   public void insertDocument(String path,
                              String name,
                              String doc)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);

      XMLResource document = (XMLResource) col.createResource(name, "XMLResource");
      document.setContent(doc);
      col.storeResource(document);
   }

   public int countDocument(String path)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);

      return col.getResourceCount();
   }

   public String getDocument(String path,
                             String name)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      XMLResource document = (XMLResource) col.getResource(name);

      if (document == null) {
         return null;
      }

      return document.getContent().toString();
   }

   public String[] listDocuments(String path)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      return col.listResources();
   }

   public void removeDocument(String path,
                              String name)
         throws Exception {
      Collection col = DatabaseManager.getCollection(driver + "/" + path);
      XMLResource document = (XMLResource) col.getResource(name);

      col.removeResource(document);
   }

}
